/*

    Copyright (C) 2003 Bjorn Martins Paz (bmp@narpax.com.br)
    Copyright (C) 2003 Davi Einstein Melges Arnaut (dema@narpax.com.br)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TEximon.h"
#include "TQueue.h"
#include "TMsgBox.h"
#include "TString.h"
#include "TAdvMsgBox.h"
#include "TClockView.h"

/* A number for each window */
const int WinQueue_Number   = 1;
const int WinMainLog_Number = 2;
const int WinGraphs_Number  = 3;
const int WinSummary_Number = 4;

/*----------------------------------------------------------------------------------------------*/
/* Constructor */
TEximon::TEximon(void) : TProgInit( &TEximon::initStatusLine, &TEximon::initMenuBar, &TEximon::initDeskTop ) {
   m_bWinQueue = false;
   m_bWinMainLog = false;
   m_bWinGraphs = false;
   m_bWinSummary = false;
    
   TRect r;
   
   /* Create and insert the clock */
   r = getExtent();
   r.a.x = r.b.x - 9;
   r.b.y = r.a.y + 1;
   clock = new TClockView ( r );
   insert (clock);
   
}

/*----------------------------------------------------------------------------------------------*/
/* Create the StatusLine */
TStatusLine *TEximon::initStatusLine(TRect r) {
   r.a.y = r.b.y - 1;     // move top to 1 line above bottom
   return new TStatusLine( r, *new TStatusDef( 0, 0xFFFF ) +            // set range of help contexts
                              *new TStatusItem( 0, kbF10, cmMenu ) +
                              *new TStatusItem( Status_Exit,  kbAltX,  cmQuit  ) +
                              *new TStatusItem( Status_Close, kbAltF4, cmClose ) );
}

/*----------------------------------------------------------------------------------------------*/
/* Create the MenuBar */
TMenuBar *TEximon::initMenuBar( TRect r ) {
   r.b.y = r.a.y + 1;   /* set bottom line 1 line below top line */
   return new TMenuBar( r, 
                           /* Windows SubMenu */
                           *new TSubMenu( SubMenu_Window, keySubMenu_Window ) +
                                *new TMenuItem( MenuItem_Queue,   cmMenuQueue,   kbF2, hcNoContext, "F2" ) +
                                *new TMenuItem( MenuItem_MainLog, cmMenuMainLog, kbF3, hcNoContext, "F3" ) +
                             // *new TMenuItem( MenuItem_Graphs,  cmMenuGraphs,  kbF4, hcNoContext, "F4" ) +
                                *new TMenuItem( MenuItem_Cmd, cmMenuCmd, kbF5, hcNoContext, "F5" ) +
                           /* Utilities SubMenu */
                           *new TSubMenu ( SubMenu_Utilities, keySubMenu_Utilities ) +
                                *new TMenuItem( MenuItem_Summary,     cmMenuSummary,     kbNoKey ) +
                                *new TMenuItem( MenuItem_Stats,       cmMenuStats,       kbNoKey ) +
                                *new TMenuItem( MenuItem_Exiwhat,     cmMenuExiwhat,     kbNoKey ) +
                                *new TMenuItem( MenuItem_CycleLogs,   cmMenuCycleLogs,   kbNoKey ) +
                                *new TMenuItem( MenuItem_CheckAccess, cmMenuCheckAccess, kbNoKey ) +
                                *new TMenuItem( MenuItem_Exigrep,     cmMenuExigrep,     kbNoKey ) +
                                *new TMenuItem( MenuItem_Exinext,     cmMenuExinext,     kbNoKey ) +
                           /* Options SubMenu */
                           *new TSubMenu ( SubMenu_Options, keySubMenu_Options ) +
                             // *new TMenuItem( MenuItem_Shell,   cmMenuShell,   kbNoKey ) +
                                *new TMenuItem( MenuItem_Color,   cmMenuColor,   kbNoKey ) +
                           /* Help SubMenu */
                           *new TSubMenu( SubMenu_Help, keySubMenu_Help ) +
                                *new TMenuItem( MenuItem_About,   cmMenuAbout,   kbF1, hcNoContext, "F1" ) );
}

/*----------------------------------------------------------------------------------------------*/
/* Event handler */
void TEximon::handleEvent( TEvent& event ) {
   if( event.what == evCommand ) {
      switch( event.message.command ) {

         /* Windows SubMenu */
         case cmMenuQueue:
            QueueWindow();
            break;
         case cmMenuMainLog:
            MainlogWindow();
            break;
         case cmMenuGraphs:
            break;
         case cmMenuCmd:
            CmdWindow();
            break;
         
         /* Utilities SubMenu */
         case cmMenuSummary:
            ExecuteCommand(SummaryTitle, ShellCmdBinary, 3, " -bp | ", ShellBinaryPath, ShellCmdSummary);
            break;
         case cmMenuStats:
            ExecuteCommand(StatsTitle, ShellCmdStats, 2, " -nr ", ShellPathMainlog);
            break;
         case cmMenuExiwhat:
            ExecuteCommand(ExiwhatTitle, ShellExiwhatPath, 0);
            break;
         case cmMenuCycleLogs:
            ExecuteCommand(CycleLogsTitle, ShellCmdCycleLogs, 0);
            break;
         case cmMenuCheckAccess:
            CheckAccessWindow();
            break;
         case cmMenuExigrep:
            ExigrepWindow();
            break;
         case cmMenuExinext:
            ExinextWindow();
            break;
         
         /* Options SubMenu */
         /* case cmMenuShell:
             Shell();
             break;
         */
         case cmMenuColor:
            Colors();
            break;
         
         /* About SubMenu */
         case cmMenuAbout:
            AboutDialog();
            break;
         
         /* Update colors */
         case cmUpdateColorsChanged:
         case cmColorsChanged:
            CacheColors();
            break;

         /* Default */   
         default:
            goto ToUpperHandler;
      };

      clearEvent(event);
      return;
   }

ToUpperHandler:
   TApplication::handleEvent(event);
   clearEvent(event);
}

/*----------------------------------------------------------------------------------------------*/
/*                                       Windows SubMenu                                        */
/*----------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------*/
/* Queue Window */
void TEximon::QueueWindow(void) {
   TQueue *queue;
   TRect rBounds(0, 0, QueueWidth, QueueHeight);

   if( m_bWinQueue )
      return;
   m_bWinQueue = true;

   queue = new TQueue(&m_bWinQueue, rBounds, WinQueue_Title);
   if( queue != NULL )
      deskTop->insert(queue);
}

/*----------------------------------------------------------------------------------------------*/
/* Mainlog Window (Temporary) */
void TEximon::MainlogWindow(void) {
   TString cmd;
   
   cmd += "tail -n 10 ";
   cmd += ShellPathMainlog;
   cmd += " 2>&1";

   ExecuteRawCommand(cmd.getData(), MainglogTitle);
   
   return;
} 

/*----------------------------------------------------------------------------------------------*/
/*                                       Utilities SubMenu                                      */
/*----------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------*/
/* Check Access Window */
bool TEximon::CheckAccessWindow(void) {
   TString cmd;
   bool Result = false;
    
   struct DialogData {
      char ip_address[128];
      char email_address[128];
      char exim_options[128];
   };

   DialogData *CheckAcessDialog = new DialogData;
   TDialog *CheckAccess = new TDialog ( TRect( 20, 4, 60, 18), CheckAccesTitle );

   if ( CheckAccess ) {
      TView *dd;
        
      /* Put something on the input */
      strcpy ( CheckAcessDialog->ip_address, "" );
      strcpy ( CheckAcessDialog->email_address, "" );
      strcpy ( CheckAcessDialog->exim_options, "" );
        
      /* IP address input */
      dd = new TInputLine ( TRect ( 3, 3, 37, 4 ), 128 );
      CheckAccess->insert ( dd );
      CheckAccess->insert ( new TLabel( TRect( 2, 2, 24, 3 ), TL_IPaddr, dd ));

      /* Email address input */        
      dd = new TInputLine ( TRect ( 3, 6, 37, 7 ), 128 );
      CheckAccess->insert ( dd );
      CheckAccess->insert ( new TLabel( TRect( 2, 5, 24, 6 ), TL_EMailaddr, dd ));

      /* Exim options input */
      dd = new TInputLine( TRect ( 3, 9, 37, 10 ), 128 );
      CheckAccess->insert ( dd );
      CheckAccess->insert ( new TLabel( TRect( 2, 8, 24, 9 ), TL_EximOpt, dd ));
        
      /* OK and Cancel buttons */
      CheckAccess->insert ( new TButton( TRect( 8, 11, 18, 13 ), OkButton, cmOK, bfDefault ));
      CheckAccess->insert ( new TButton( TRect( 20, 11, 30, 13 ), CancelButton, cmCancel, bfNormal ));
        
      /* Saving the dialog data */
      CheckAccess->setData ( CheckAcessDialog );

      ushort button = deskTop->execView ( CheckAccess );

      /* Read it back */
      if ( button != cmCancel ) {
         CheckAccess->getData ( CheckAcessDialog );
                
         cmd += ShellBinaryPath;
         cmd += ShellCmdCheckAccess;
         cmd += " ";
         cmd += CheckAcessDialog->ip_address;
         cmd += " ";
         cmd += CheckAcessDialog->email_address;
         cmd += " ";
         cmd += CheckAcessDialog->exim_options;
         cmd += " 2>&1";

         Result = ExecuteRawCommand(cmd.getData(), CheckAccesTitle);
            
         cmd.clean();
      }
   }
        
   destroy ( CheckAccess );

   return Result;
}

/*----------------------------------------------------------------------------------------------*/
/* exigrep Window */
bool TEximon::ExigrepWindow (void) {
    TString cmd;

    struct DialogData
    {
        char pattern[128];
        char log_file[128];
        int literal;
    };

    DialogData *ExigrepDialog = new DialogData;

    TDialog *Exigrep = new TDialog( TRect( 20, 4, 60, 17), ExigrepTitle );

    if ( Exigrep ) {

        TView *dd;

        /* Put something on the input */
        strcpy ( ExigrepDialog->pattern, "" );
        strcpy ( ExigrepDialog->log_file, ShellPathMainlog );
        ExigrepDialog->literal = 0;
        
        /* Pattern */
        dd = new TInputLine( TRect ( 3, 3, 37, 4 ), 128 );
        Exigrep->insert ( dd );
        Exigrep->insert ( new TLabel( TRect( 2, 2, 24, 3 ), TL_Pattern, dd ));

        /* Logfile */
        dd = new TInputLine( TRect ( 3, 6, 37, 7 ), 128 );
        Exigrep->insert ( dd );
        Exigrep->insert ( new TLabel ( TRect( 2, 5, 24, 6 ), TL_LogFile, dd ));

        /* Literal */
        dd = new TCheckBoxes( TRect ( 3, 8, 18, 9 ), new TSItem( TL_Literal, 0 ) );
        Exigrep->insert ( dd );

        /* OK and Cancel buttons */
        Exigrep->insert ( new TButton( TRect ( 8, 10, 18, 12 ), OkButton, cmOK, bfDefault ));
        Exigrep->insert ( new TButton( TRect ( 20, 10, 30, 12 ), CancelButton, cmCancel, bfNormal ));

        /* Saving the dialog data */
        Exigrep->setData ( ExigrepDialog );

        ushort button = deskTop->execView( Exigrep );

        /* Read it back */
        if ( button != cmCancel ) {

            Exigrep->getData ( ExigrepDialog );
            
            if ( strlen(ExigrepDialog->pattern) <= 0 || strlen(ExigrepDialog->log_file) <= 0)
                return false;

            cmd += ShellBinaryPath;
            cmd += ShellCmdExigrep;
            cmd += " ";
            
            if (ExigrepDialog->literal == 1)            
                cmd += " -l ";
            
            cmd += " ";
            cmd += ExigrepDialog->pattern;
            cmd += " ";
            cmd += ExigrepDialog->log_file;
            cmd += " 2>&1";

            ExecuteRawCommand(cmd.getData(), ExigrepTitle);

            cmd.clean();

        }
    }

    destroy ( Exigrep );
    return true;
}

/*----------------------------------------------------------------------------------------------*/
/* Exinext Window */
bool TEximon::ExinextWindow (void) {
   TString cmd;

   struct DialogData {
      char pattern[128];
   };

   DialogData *ExinextDialog = new DialogData;
   TDialog *Exinext = new TDialog( TRect( 20, 4, 60, 12), ExinextTitle );

   if ( Exinext ) {
      TView *dd;

      /* Put something on the input */
      strcpy ( ExinextDialog->pattern, "" );

      /* Pattern */
      dd = new TInputLine( TRect ( 3, 3, 37, 4 ), 128 );
      Exinext->insert ( dd );
      Exinext->insert ( new TLabel( TRect( 2, 2, 37, 3 ), TL_Addrs, dd ));

      /* OK and Cancel buttons */
      Exinext->insert ( new TButton( TRect ( 8, 5, 18, 7 ), OkButton, cmOK, bfDefault ));
      Exinext->insert ( new TButton( TRect ( 20, 5, 30, 7 ), CancelButton, cmCancel, bfNormal ));

      /* Saving the dialog data */
      Exinext->setData ( ExinextDialog );

      ushort button = deskTop->execView ( Exinext );

      /* Read it back */
      if ( button != cmCancel ) {

         Exinext->getData ( ExinextDialog );

         if ( strlen(ExinextDialog->pattern) <= 0)
            return false;

         cmd += ShellBinaryPath;
         cmd += ShellCmdExinext;
         cmd += " ";
         cmd += ExinextDialog->pattern;
         cmd += " 2>&1";

         ExecuteRawCommand(cmd.getData(), ExinextTitle);

         cmd.clean();
      }
   }

   destroy( Exinext );
   return true;
}

/*----------------------------------------------------------------------------------------------*/
/* CMD Window */
bool TEximon::CmdWindow (void) {
   TString cmd;

   struct DialogData {
      char cmd[128];
   };

   DialogData *CmdDialog = new DialogData;
   TDialog *CmdD = new TDialog( TRect( 20, 4, 60, 12), CommandTitle );

   if ( CmdD ) {
      TView *dd;

      /* Put something on the input */
      strcpy ( CmdDialog->cmd, "" );

      /* Pattern */
      dd = new TInputLine( TRect ( 3, 3, 37, 4 ), 128 );
      CmdD->insert ( dd );
      CmdD->insert ( new TLabel( TRect( 2, 2, 37, 3 ), TL_Command, dd ));

      /* OK and Cancel buttons */
      CmdD->insert ( new TButton( TRect ( 8, 5, 18, 7 ), OkButton, cmOK, bfDefault ));
      CmdD->insert ( new TButton( TRect ( 20, 5, 30, 7 ), CancelButton, cmCancel, bfNormal ));

      /* Saving the dialog data */
      CmdD->setData ( CmdDialog );

      ushort button = deskTop->execView ( CmdD );

      /* Read it back */
      if ( button != cmCancel ) {

         CmdD->getData ( CmdDialog );

         if ( strlen(CmdDialog->cmd) <= 0)
            return false;

         cmd += CmdDialog->cmd;
         cmd += " 2>&1";

         ExecuteRawCommand(cmd.getData(), CommandTitle);

         cmd.clean();
      }
   }

   destroy( CmdD );
   
   return true;
}


/*----------------------------------------------------------------------------------------------*/
/*                                        Options SubMenu                                       */
/*----------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------*/
/* Shell */
/*
void TEximon::Shell(void) {

   if (TScreen::noUserScreen())
      return;

   suspend();
   TScreen::System(CLY_GetShellName());
   resume();
   redraw();

}
*/
/*----------------------------------------------------------------------------------------------*/
/* Colors */
static TPalette *temp_pallet = 0;
static int colorsCached;

#define NumberColors 7

static TColorDialog *GetColorDialog() {
    TColorDialog *ColorDialog;

    TColorGroup &group1 =  *new TColorGroup("Desktop") +
                              *new TColorItem("Color",             1) +
                           *new TColorGroup("Menus") +
                              *new TColorItem("Normal",            2) +
                              *new TColorItem("Disabled",          3) +
                              *new TColorItem("Shortcut",          4) +
                              *new TColorItem("Selected",          5) +
                              *new TColorItem("Selected disabled", 6) +
                              *new TColorItem("Shortcut selected", 7);

    ColorDialog = new TColorDialog(&TProgram::application->getPalette(), &group1);
    temp_pallet = new TPalette(TProgram::application->getPalette());
    ColorDialog->setData(&TProgram::application->getPalette());

    return ColorDialog;
}
                         
/*----------------------------------------------------------------------------------------------*/
/* Color Control Dialog Box function */
void TEximon::Colors(void) {
   TColorDialog *ColorDialog = GetColorDialog();

   if ( TProgram::application->validView( ColorDialog ) != 0) {
      if ( TProgram::deskTop->execView( ColorDialog ) == cmCancel )
         TProgram::application->getPalette() = *temp_pallet;

      colorsCached = 0;
      TProgram::application->Redraw();
      destroy (ColorDialog);
   }

   delete temp_pallet;

   temp_pallet = 0;
}

/*----------------------------------------------------------------------------------------------*/
/* Cache the colors */
static uchar cachedColors[NumberColors];

void TEximon::CacheColors() {
    int i;
    
    for ( i = 1 ; i < NumberColors ; i++ )
        cachedColors[i] = getColor (i);
}

/*----------------------------------------------------------------------------------------------*/
/*                                           Help SubMenu                                       */
/*----------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------*/
/* About Dialog */
void TEximon::AboutDialog(void) {
   TMsgBox *about;

   about = new TMsgBox( WinAbout_Title, WinAbout_Text );
   if( about != NULL ) {
      if( validView(about) != 0 ) {
         deskTop->execView(about);
         destroy(about);
      }
   }
}

/*----------------------------------------------------------------------------------------------*/
/*                                  Various Calls (change my name..)                            */
/*----------------------------------------------------------------------------------------------*/
/* Execute Command */
bool TEximon::ExecuteCommand(const char *sTitle, const char *sBin, int nvars, ...) {
   TString sCmd;
   bool Result;
   int n;
   va_list ap;
   va_start (ap, nvars);
      
   sCmd.clean();
   sCmd += ShellBinaryPath;
   sCmd += sBin;
   
   for (n = 0; n < nvars; ++n)
        sCmd += va_arg (ap, const char *);
   
   sCmd += " 2>&1";

   Result = ExecuteRawCommand(sCmd.getData(), sTitle);

   sCmd.clean();
   return Result;
}

/*----------------------------------------------------------------------------------------------*/
/* Execute Raw Command */
bool TEximon::ExecuteRawCommand(const char *sBin, const char *sTitle) {
   char ch;
   FILE *shell;
   TString sOutput;
   TAdvMsgBox *mbOutput;

   /* Execute the command line */
   if( (shell = popen(sBin, "r")) <= 0 )
      return false;

   /* Get the output from the shell and store it */
   sOutput.clean();
   ch = fgetc(shell);
   
   while( !feof(shell) ) {
      sOutput += ch;
      ch = fgetc(shell);
   }
   
   pclose(shell);

   /* Show the console output in a Message Box */
   if( sOutput.getData() != NULL ) {
      mbOutput = new TAdvMsgBox( sTitle, sOutput.getData() );
      if( mbOutput != NULL )
         TProgram::deskTop->insert(mbOutput);
   }

   sOutput.clean();

   return true;
}

/*----------------------------------------------------------------------------------------------*/
/* isTileable() function ( checks a view on desktop is tileable or not ) */
static Boolean isTileable(TView *p, void * ) {
   if( (p->options & ofTileable) != 0)
       return True;
   else
       return False;
}

/*----------------------------------------------------------------------------------------------*/
/* idle() function */
void TEximon::idle() {
    TProgram::idle();
 
    /* Update the clock */
    clock->update();

    if (deskTop->firstThat(isTileable, 0) != 0 ) {
        enableCommand (cmTile);
        enableCommand (cmCascade);
    } else {
        disableCommand (cmTile);
        disableCommand (cmCascade);
    }
}



