/*

    Copyright (C) 2003 Bjorn Martins Paz (bmp@narpax.com.br)
    Copyright (C) 2003 Davi Einstein Melges Arnaut (dema@narpax.com.br)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TInputBox.h"

#define MAX_INPUTSIZE 0xFF
#define TInputBoxWidth 40

/*----------------------------------------------------------------------------------------------*/
/* Constructor */
TInputBox::TInputBox(char **psInput, const char *sTitle )
         : TDialog( TRect(0,0,TInputBoxWidth,6), sTitle), TWindowInit(&TInputBox::initFrame) {

   this->options |= ofCentered;

   m_psInput  = psInput;
   *m_psInput = NULL;

   TRect rWin = getExtent();
   TRect rInp = TRect(2, 1, rWin.b.x-2, 2);
   TRect rButOK     = TRect(rWin.b.x/3-5, rWin.b.y-3, rWin.b.x/3+5, rWin.b.y-1);
   TRect rButCancel = TRect( ((rWin.b.x>>1)-6) + (rWin.b.x/3-6), rWin.b.y-3,
                             ((rWin.b.x>>1)-6) + (rWin.b.x/3+6), rWin.b.y-1);

   m_InputLine = new TInputLine( rInp, MAX_INPUTSIZE );
   this->insert( m_InputLine );
   this->insert( new TButton( rButOK, OkButton, cmOK, bfDefault) );
   this->insert( new TButton( rButCancel, CancelButton, cmCancel, bfDefault) );
}

/*----------------------------------------------------------------------------------------------*/
/* Handle Event */
void TInputBox::handleEvent(TEvent& event) {
   char *temp;

   if( (event.what == evCommand && event.message.command == cmOK) ||
       (event.what == evKeyDown && event.keyDown.keyCode == kbEnter) ) {

      /* Check if the user typed something */
      if( strlen(m_InputLine->data) > 0 ) {
         temp = new char[ strlen(m_InputLine->data) + 1 ];
         temp[0] = '\0';

         /* Get rid of all leading and tailing spaces using sscanf() */
         sscanf(m_InputLine->data,"%s",temp);

         /* Check again to see if the user typed something usefull (not just spaces) */
         if( strlen(temp) > 0 ) {
            /* Ok, there was something, let's copy it */
            *m_psInput = new char[ strlen(m_InputLine->data) + 1 ];
            strcpy(*m_psInput, m_InputLine->data);
         }
      }
   }

   TDialog::handleEvent(event);
}
