/*

    Copyright (C) 2003 Bjorn Martins Paz (bmp@narpax.com.br)
    Copyright (C) 2003 Davi Einstein Melges Arnaut (dema@narpax.com.br)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TParseConfig.h"

// Keep these numbers equal!!!
#define MAX_SIZE_INT 50
#define MAX_SIZE_STR "50"

// Exim default options
char *ShellPathMainlog      = "/var/spool/exim/log/mainlog";
char *ShellBinaryPath       = "/usr/exim/bin/"; // Note: the final '/' character IS NECESSARY!!!
char *ShellCmdBinary        = "exim";
char *ShellCmdSummary       = "exiqsumm";
char *ShellCmdStats         = "eximstats";
char *ShellExiwhatPath      = "exiwhat";
char *ShellCmdCycleLogs     = "exicyclog";  
char *ShellCmdCheckAccess   = "exim_checkaccess";
char *ShellCmdExigrep       = "exigrep";
char *ShellCmdExinext       = "exinext";

static struct OPTIONLIST {
  char   *name;
  char   **value;
} copt[] = { // Config Options
   { "mainlog_path"    , &ShellPathMainlog    },
   { "bin_path"        , &ShellBinaryPath     },
   { "exim_bin"        , &ShellCmdBinary      },
   { "exiqsumm_bin"    , &ShellCmdSummary     },
   { "eximstats_bin"   , &ShellCmdStats       },
   { "exiwhat_bin"     , &ShellExiwhatPath    },
   { "exicyclog_bin"   , &ShellCmdCycleLogs   },
   { "checkaccess_bin" , &ShellCmdCheckAccess },
   { "exigrep_bin"     , &ShellCmdExigrep     },
   { "exinext_bin"     , &ShellCmdExinext     }
};

static int OptionsCount = sizeof(copt) / sizeof(OPTIONLIST);

void TParseConfig::Parse(void) {
   char *p_name, *p_value;
   FILE *fd;
   int i;
    
   p_name  = new char [MAX_SIZE_INT + 1];
   p_value = new char [MAX_SIZE_INT + 1];
    
   fd = fopen (CONFIG_FILE, "r");

   if (fd == NULL)
      return;

   while ((i = fscanf(fd, "%"MAX_SIZE_STR"s = %"MAX_SIZE_STR"s", p_name, p_value)) != EOF) {
      // Ignore lines starting with '#', its a comment
      if (p_name[0] == '#')
         continue;

      // Check if the option exists. If true, apply new value
      for( i = 0; i < OptionsCount; i++ ) {
         if ( ! strcmp(p_name, copt[i].name) ) {
            *copt[i].value = strdup(p_value);
            break;
         }
      }
   }

   if (fd)
      fclose(fd);
        
   delete p_name;
   delete p_value;
        
   return;
}

