/*

    Copyright (C) 2003 Bjorn Martins Paz (bmp@narpax.com.br)
    Copyright (C) 2003 Davi Einstein Melges Arnaut (dema@narpax.com.br)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TQueue.h"
#include "TString.h"
#include "TInputBox.h"
#include "TAdvMsgBox.h"
#include "TCheckBoxWindow.h"

#define BUFFER_SIZE  512
#define MAX_SUBSIZE  255

/*----------------------------------------------------------------------------------------------*/
/* Constructor */
TQueue::TQueue(bool *pbFlag, const TRect& rBounds, const char *sTitle)
      : TDialog(rBounds, sTitle), TWindowInit(&TQueue::initFrame) {

   /* Keep the pointer to the flag that indicates that this window exists */
   m_pbExistenceFlag = pbFlag;

   TRect r = getExtent();

   this->flags = wfMove | wfClose | wfGrow | wfZoom;

   m_MailList = new TNSCollection(0,1);

   /* Create the vertical and horizontal scrollbar rectangles.
      They will be put on top of the window borders. */
   TRect r_vs(r.b.x-1, r.a.y+1, r.b.x, r.b.y-1);
   TRect r_hs(r.a.x+1, r.b.y-1, r.b.x-1, r.b.y);

   /* Create the scrollbars. */
   m_vScrollBar = new TScrollBar(r_vs);
   m_hScrollBar = new TScrollBar(r_hs);

   /* Make the listbox fit exactly INSIDE the window
      (not overlapping the window borders). */
   r.grow(-1,-1);

   /* Create the listbox. */
   m_ListBox = new TListBox(r, 1, m_hScrollBar, m_vScrollBar);
   m_ListBox->growMode = gfGrowHiX | gfGrowHiY;

   this->insert(m_hScrollBar);
   this->insert(m_vScrollBar);
   this->insert(m_ListBox);

   this->RefreshQueueList();
}

/*----------------------------------------------------------------------------------------------*/
/* Destructor */
TQueue::~TQueue() {
   m_MailList->freeAll();
   delete m_MailList;

   /* Tell the main application that this window does not exist anymore */
   *m_pbExistenceFlag = false;
}

/*----------------------------------------------------------------------------------------------*/
/* Event Handler */
void TQueue::handleEvent(TEvent& event) {
   bool MouseAction = false;
   TListBoxRec rec;
   TEvent ev;
   int id;
  
   /* */
   if( (event.what == evMouseDown && event.mouse.buttons == mbRightButton) ) {
      ev.what = evMouseDown;
      ev.message.command = event.message.command;
      ev.message.infoPtr = event.message.infoPtr;
      ev.mouse.where.x = event.mouse.where.x;
      ev.mouse.where.y = event.mouse.where.y;
      ev.mouse.buttons = mbLeftButton;
      TDialog::handleEvent(ev);
      ev.what = evMouseUp;
      ev.message.command = event.message.command;
      ev.message.infoPtr = event.message.infoPtr;
      ev.mouse.where.x = event.mouse.where.x;
      ev.mouse.where.y = event.mouse.where.y;
      ev.mouse.buttons = mbLeftButton;
      TDialog::handleEvent(ev);
      MouseAction = true;
   }

   if( (event.what == evKeyDown && event.keyDown.keyCode == kbEnter) || MouseAction ) {
      m_ListBox->getData(&rec);
      id = rec.selection;
      switch( PopupBox(event) ) {
         case cmMBDeliver:
            EximSimpleCommand(ShellCmdDeliver, "2>&1", QueueTitleDeliver, id);
            break;
         case cmMBFreeze:
            EximSimpleCommand(ShellCmdFreeze, "2>&1", QueueTitleFreeze, id);
            break;
         case cmMBThaw:
            EximSimpleCommand(ShellCmdThaw, "2>&1", QueueTitleThaw, id);
            break;
         case cmMBMarkAll:
            EximSimpleCommand(ShellCmdMarkAll, "2>&1", QueueTitleMarkAll, id);
            break;
         case cmMBReturn:
            EximSimpleCommand(ShellCmdReturn, "2>&1", QueueTitleReturn, id);
            break;
         case cmMBCancel:
            EximSimpleCommand(ShellCmdRemove, "2>&1", QueueTitleCancel, id);
            break;
         case cmMBHeader:
            EximSimpleCommand(ShellCmdHeader, "2>&1", QueueTitleHeader, id);
            break;
         case cmMBBody:
            EximSimpleCommand(ShellCmdBody, "2>&1", QueueTitleBody, id);
            break;
         case cmMBAddRcpt:
            CommandWithInput(ShellCmdAddRcpt, "2>&1", QueueTitleAddRcpt, id);
            break;
         case cmMBMark:
            EximMarkRecipients(ShellCmdMark, "2>&1", QueueTitleMark, id);
            break;
         case cmMBAddSender:
            CommandWithInput(ShellCmdAddSender, "2>&1", QueueTitleAddSender, id);
            break;
         case cmMBLogSpool:
            EximSimpleCommand(ShellCmdLogSpool, "2>&1", QueueTitleLogSpool, id);
            break;
         default:;
            clearEvent(event);
            return;
      };
      clearEvent(event);
      this->RefreshQueueList();
      return;
   }

   TDialog::handleEvent(event);
   clearEvent(event);
}

/*----------------------------------------------------------------------------------------------*/
/* Popup Box */
int TQueue::PopupBox(TEvent& event) {
   unsigned int cmMBOption;

   TRect pr;
   TRect dr = TProgram::deskTop->getExtent();
   TRect wr = getBounds();

   if( event.what == evMouseDown ) {
      pr.a.x = event.mouse.where.x-2;
      pr.a.y = event.mouse.where.y-1;
      pr.b.x = dr.b.x;
      pr.b.y = dr.b.y;
   }
   else {
      pr.a.x = wr.b.x-2;
      pr.a.y = wr.a.y+1;
      pr.b.x = dr.b.x;
      pr.b.y = dr.b.y;
   }

   TMenu *menu = new TMenu( *new TMenuItem(QueueMBDeliver,   cmMBDeliver,   0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBFreeze,    cmMBFreeze,    0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBThaw,      cmMBThaw,      0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBMarkAll,   cmMBMarkAll,   0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBReturn,    cmMBReturn,    0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBCancel,    cmMBCancel,    0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBHeader,    cmMBHeader,    0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBBody,      cmMBBody,      0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBAddRcpt,   cmMBAddRcpt,   0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBMark,      cmMBMark,      0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBAddSender, cmMBAddSender, 0, hcNoContext, (char *)0, +
                             new TMenuItem(QueueMBLogSpool,  cmMBLogSpool,  0, hcNoContext, (char *)0)
                          ))))))))))));

   TMenuBox *m_MenuBox = new TMenuBox(pr, menu, (TMenuView *)0);
   cmMBOption = TProgram::deskTop->execView(m_MenuBox);
   this->destroy(m_MenuBox);

   return cmMBOption;
}

/*----------------------------------------------------------------------------------------------*/
/* Refresh Queue List */
void TQueue::RefreshQueueList(void) {
   TString sTemp;
   EMAIL *mail;
   TStringCollection *list;
   ccIndex mail_index, rcpt_index;
   int list_index = 0;

   list = new TStringCollection(0,1);
   list->freeAll();

   if( !this->GetQueueList(ShellCmdListQueue) ) {
      list->insert( newStr("Error getting queue list") );
      m_ListBox->newList(list);
   }

   for( mail_index = 0; mail_index < m_MailList->getCount(); mail_index++ ) {
      mail = (EMAIL *) m_MailList->at(mail_index);
      sTemp.clean();
      if( mail->frozen )
         sTemp += QueueFlagFrozen;
      else
         sTemp += QueueFlagNone;
      sTemp += " ";
      sTemp += mail->time;   sTemp += " ";
      sTemp += mail->size;   sTemp += " ";
      sTemp += mail->sender; sTemp += " ";

      for( rcpt_index = 0; rcpt_index < mail->Rcpts->getCount(); rcpt_index++ ) {
         sTemp += (char *) mail->Rcpts->at(rcpt_index);
         sTemp += " ";
      }
     
      list->atInsert( list_index++, sTemp.cutData() );
   }

   m_ListBox->newList(list);
}

/*----------------------------------------------------------------------------------------------*/
/* Get Queue List */
bool TQueue::GetQueueList(const char *sCommand) {
   EMAIL *mail;
   char *buffer, *temp_time, *temp_size, *temp_id, *temp_sender, *temp_recipient, *temp;
   FILE *shell;
   TString cmd;
   int rcpt_index, mail_index = 0;
               
   cmd += ShellBinaryPath;
   cmd += ShellCmdBinary;
   cmd += " ";
   cmd += sCommand;

   if( (shell = popen(cmd.getData(), "r")) <= 0 )
      return false;

   buffer         = new char[BUFFER_SIZE];
   temp_time      = new char[MAX_SUBSIZE];
   temp_size      = new char[MAX_SUBSIZE];
   temp_id        = new char[MAX_SUBSIZE];
   temp_sender    = new char[MAX_SUBSIZE];
   temp_recipient = new char[MAX_SUBSIZE];

   m_MailList->freeAll();
   
   while( true ) {
      buffer[0] = '\0';
      fgets(buffer, BUFFER_SIZE, shell);

      /* Check if there was something in the queue */
      if( strlen(buffer) == 0 )
         break;

      /* Allocate a new email struture */
      mail = new EMAIL;
      mail->frozen = false;
      mail->Rcpts = new TStringCollection(0,1);
      mail->Rcpts->freeAll();

      /* Check if this mail is frozen */
      if( strstr(buffer, MailFlagFrozen) )
         mail->frozen = true;

      /* Extract the tokens of the mail-line */
      if( sscanf(buffer, "%s %s %s %s", temp_time, temp_size, temp_id, temp_sender) == 0 )
         break;

      /* Allocate the exact space needed for each token */
      mail->time   = new char[ strlen(temp_time)   + 1 ];
      mail->size   = new char[ strlen(temp_size)   + 1 ];
      mail->id     = new char[ strlen(temp_id)     + 1 ];
      mail->sender = new char[ strlen(temp_sender) + 1 ];

      /* Copy all tokens to our structure */
      strcpy( mail->time,   temp_time   );
      strcpy( mail->size,   temp_size   );
      strcpy( mail->id,     temp_id     );
      strcpy( mail->sender, temp_sender );

      rcpt_index = 0;

      /* Read all recipients until we find a blank line */
      while( true ) {
         buffer[0] = '\0';
         fgets(buffer, BUFFER_SIZE, shell);

         /* Try to get a recipient */
         if( sscanf(buffer, "%s", temp_recipient) != 1 )
            break;

         /* Check if we got a "mark as delivered" flag instead
            of a valid recipient address */
         if( !strcmp(temp_recipient, RcptFlagDelivered) )
            continue;

         /* Insert recipient in the list */
         temp = new char[ strlen(temp_recipient) + 1 ];
         strcpy( temp, temp_recipient );
         mail->Rcpts->atInsert( rcpt_index++, temp );
      }

      m_MailList->atInsert( mail_index++, mail );
   }

   pclose(shell);

   delete buffer;        
   delete temp_time;     
   delete temp_id;       
   delete temp_sender;   
   delete temp_recipient;

   return true;
}

/*----------------------------------------------------------------------------------------------*/
/* Exim Simple Command */
bool TQueue::EximSimpleCommand(const char *sBin, const char *sExtra, const char *sTitle, int id) {
   EMAIL *mail;
   TString sCmd;
   bool Result;

   if( id >= m_MailList->getCount() )
      return false;

   mail = (EMAIL *) m_MailList->at(id);

   /* Build commandline to be executed */
   sCmd.clean();
   sCmd += ShellBinaryPath; 
   sCmd += ShellCmdBinary;
   sCmd += " ";
   sCmd += sBin;
   sCmd += " ";
   sCmd += mail->id;
   if( sExtra != NULL ) {
      sCmd += " ";
      sCmd += sExtra;
   }

   Result = ExecuteCommand(sCmd.getData(), sTitle);

   sCmd.clean();
   return Result;
}

/*----------------------------------------------------------------------------------------------*/
/* Exim Mark Recipients */
bool TQueue::EximMarkRecipients(const char *sCmd, const char *sExtra, const char *sTitle, int id) {
   EMAIL *mail;
   TString cmd;
   int index = 0;
   bool status = false;
   TStringCollection *list;
   TCheckBoxWindow *dialog;
   bool at_least_one = false;
   unsigned int selection, mask;


   /* Check if 'id' is a valid position in m_MailList */
   if( id >= m_MailList->getCount() )
      return false;

   /* Get the EMAIL structure from m_MailList at position 'id' */
   mail = (EMAIL *) m_MailList->at(id);

   /* Check if there is at least one recipient available to be marked */
   if( mail->Rcpts->getCount() == 0 )
      return false;

   /* Create the window with the checkboxes. Every bit in 'selection'
      represents a recipient. If a bit is set (1) then the recipient
      that that position was marked and should be included in the shell
      command as parameter */
   dialog = new TCheckBoxWindow( &selection, sTitle, mail->Rcpts );
   if( dialog != NULL ) {
      TApplication::deskTop->execView(dialog);
      destroy(dialog);
   }
   
   cmd.clean();
   cmd += ShellBinaryPath;
   cmd += ShellCmdBinary;
   cmd += " ";
   cmd += sCmd;
   cmd += " ";
   cmd += mail->id;

   list = new TStringCollection(0,1);
   list->freeAll();

   /* Find out wich recipient was marked and 
      add these to the shell command as parameters */
   mask = 1;
   for( index=0; index<mail->Rcpts->getCount(); index++ ) {
      if( selection & mask ) {
         cmd += " ";
         cmd += (char *) mail->Rcpts->at(index);
         at_least_one = true;
      }
      mask = mask << 1;
   }

   /* Execute the shell command only if at least 
      one recipient was marked */
   if( at_least_one ) {
      if( sExtra != NULL ) {
         cmd += " ";
         cmd += sExtra;
      }
      status = ExecuteCommand( cmd.getData(), sTitle );
   }

   cmd.clean();
   delete list;

   return status;
}

/*----------------------------------------------------------------------------------------------*/
/* Command With Input */
bool TQueue::CommandWithInput(const char *sBin, const char *sExtra, const char *sTitle, int id) {
   EMAIL *mail;
   TString sCmd;
   TInputBox *dialog;
   char *input = NULL;
   bool status = false;

   /* Check if 'id' is a valid position in m_MailList */
   if( id >= m_MailList->getCount() )
      return false;

   /* Get the EMAIL structure from m_MailList at position 'id' */
   mail = (EMAIL *) m_MailList->at(id);

   dialog = new TInputBox( &input, sTitle );
   if( dialog != NULL ) {
      TApplication::deskTop->execView(dialog);
      destroy(dialog);

      if( input == NULL )
         return false;

      sCmd.clean();
      sCmd += ShellBinaryPath;
      sCmd += ShellCmdBinary;
      sCmd += " ";
      sCmd += sBin;
      sCmd += " ";
      sCmd += mail->id;
      sCmd += " ";
      sCmd += input;
      if( sExtra != NULL ) {
         sCmd += " ";
         sCmd += sExtra;
      }

      delete[] input;

      status = ExecuteCommand( sCmd.getData(), sTitle );
   }

   sCmd.clean();
   return status;
}

/*----------------------------------------------------------------------------------------------*/
/* Execute Command */
bool TQueue::ExecuteCommand(const char *sCmd, const char *sTitle) {
   char ch;
   FILE *shell;
   TString output;     
   TAdvMsgBox *mbOutput;

   /* Execute the command line */
   if( (shell = popen(sCmd, "r")) <= 0 )
      return false;

   /* Get the output from the shell and store it */
   output.clean();
   ch = fgetc(shell);
   while( !feof(shell) ) {
      output += ch;
      ch = fgetc(shell);
   }
   
   pclose(shell);

   /* Show the console output in a Message Box */
   if( output.getData() != NULL ) {
      mbOutput = new TAdvMsgBox( sTitle, output.getData() );
      if( mbOutput != NULL )
         TProgram::deskTop->insert(mbOutput);
   }

   output.clean();

   return true;
}
